# $Id: set_vddk.ps1,v 1.5 2013/09/13 10:41:08 fo Exp $
# Check for existing 32 bit VMWare VDDK installation,
# extract 64 bit components and creates registry key.
#
# Needs:     
# set-executionpolicy remotesigned
#

# Parameter, which can be used, when calling script
param(
[string]$invalidarg = "",
[switch]$f,
[string]$e = "without",
[switch]$h,
[bool]$ssl = 0,
[switch]$c,
[string]$l = "nolog")

if ( $invalidarg -ne "" ) # Show help, if wrong arguments have been specified.
{
    Write-Host ""
    Write-Host "Usage of set_vddk.ps1 script:"
    Write-Host "set_vddk.ps1 [-f] [-e update|install|remove] [-ssl 0|1]"
    Write-Host "`t-f`t`tforce script to run"
    Write-Host "`t-e`t`tuse script for install, update or remove process"
    Write-Host "`t-ssl`t`tenable or disable SSLverification of VDDK (disabled by default). Just necessary in case of install and update routine."
    Write-Host "`t-c`t`tcheck installed version of VDDK."
    Write-Host "`t-l`t`twrite output to console and file, not just to console."
    Write-Host ""
    Write-Host "`t`t`tWithout parameters the script updates old VDDK structure (less or equal 5.1.1) and adjusts the sesam libraries."
    Write-Host ""
    Write-Error "At least one parameter was wrong or missing. Exiting."
    exit 9
}

# Check, whether current user has not local admin permissions
if ( ! ([Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator") )
{
    Write-Warning "Powershell is not opened with local admin rights, exiting."
    exit 7
}

if ( $l -ne "nolog" ) # Has parameter -l been specified?
{
    Start-Transcript -Append -Path $l           # Start capture of whole output to specified file
    Write-Host "Script has been called with parameter(s):"
    Write $PSBoundParameters
}

function usage()
{
    Write-Host ""
    Write-Host "Usage of set_vddk.ps1 script:"
    Write-Host "set_vddk.ps1 [-f] [-e update|install|remove] [-ssl 0|1]"
    Write-Host "`t-f`t`tforce script to run"
    Write-Host "`t-e`t`tuse script for install, update or remove process (just possible if VDDK 5.5 or higher is in use)"
    Write-Host "`t-ssl`t`tenable or disable SSLverification of VDDK (disabled by default). Just necessary in case of install and update routine."
    Write-Host "`t-c`t`tcheck installed version of VDDK."
    Write-Host "`t-l`t`twrite output to console and file, not just to console."
    Write-Host ""
    Write-Host "`t`t`tWithout parameters the script updates old VDDK structure (less or equal 5.1.1) and adjusts the sesam libraries."
    Write-Host ""
}

function copy_vddk_files ()
{
    Write-Host "$((Get-Date).ToString()): Creating structure ..."
       # Create structure in registry
	   if ( new-item -ItemType directory "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit" -force )
	   {
    		# path of installation
    		$x64_vddk_path = ((Get-ItemProperty "HKLM:\SOFTWARE\Wow6432Node\VMware, Inc.\VMware Virtual Disk Development Kit\").InstallPath) + "bin"
    	
    		# path of new folder
    		$x86_vddk_path = $env:ProgramFiles + "\VMware\VMware Virtual Disk Development Kit\"
            
    		# Create folder structure in file system
    		new-item -ItemType directory $x86_vddk_path -force
    	
    		# Create registry hive
    		New-ItemProperty -path 'HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit' -name InstallPath -PropertyType String -value $x86_vddk_path
		    New-ItemProperty -path 'HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit' -name VerifySSLCertificates -PropertyType DWord -value $sslkey

    		# Extract zip file from installation to new directory
    		$zipfilename = "$x64_vddk_path" + "\vddk64.zip";            	        # Replace with source ZIP file 
    		$destination = "$x86_vddk_path";                        			    # Replace with target folder
    	
    		if(test-path($zipfilename))                                  		# Test if the zip file exists 
    		{    
    			$shellApplication = new-object -com shell.application 
    			$zipPackage = $shellApplication.NameSpace($zipfilename) 
    			$destinationFolder = $shellApplication.NameSpace($destination)

			    $vddk_content = Get-ChildItem $x86_vddk_path | where-object { $_.GetType().Name -eq "DirectoryInfo"  }		# Get all folders in vddk-path directory
			    if ( $vddk_content.Length -ne $null )		# Skip this part, if directory contains no folder (i.e. first execution of script at all)
			    {
				    $destination_moved = $env:ProgramFiles + "\VMware\VMware Virtual Disk Development Kit" + "_org"
				    Write-Host "$((Get-Date).ToString()): Renaming folder $destination to $destination_moved"
				    Move-Item $destination $destination_moved -ErrorAction:SilentlyContinue -ErrorVariable moveerror
				    if ( $? -eq $false ) 		# Just continue, if move was successful
				    {
					   Write-Warning "$moveerror Exiting with Error 5."
                       if ( $l -ne "nolog" ) # Has parameter -l been specified?
                       {
                            Stop-Transcript
                       }
					   exit 5
				    }
				    new-item -ItemType directory $x86_vddk_path -force	# Create folder again, because original folder was renamed before
			    }

			    $destinationFolder.CopyHere($zipPackage.Items(), 0x14)         # Extract files and overwrite all files without asking (0x14)
			
			    if ( $vddk_content.Length -ne $null )		# Skip this part, if directory contains no folder before files were extracted (i.e. first execution of script at all)
			    {
				    Remove-Item $destination_moved -Force -Recurse
			    }

    		}
		    else 
		    { 
			     Write-Host "$((Get-Date).ToString()): The source ZIP file $zipfilename does not exist";
                 if ( $l -ne "nolog" ) # Has parameter -l been specified?
                  {
                        Stop-Transcript
                  }
			     exit 3
	       	}
        }
        else
	    {
	       Write-Warning "Probably no permission to create structure. Assure to login as privileged user. Exiting"
           if ( $l -ne "nolog" ) # Has parameter -l been specified?
           {
                       Stop-Transcript
           }
		   exit 2
	    }
}


function copy_sesam_libraries ($vddk_bin_path)
{
        # Save current directory
        $currentdir = (Get-Location).Path
        
        # Adjust VDDK path
        Set-Location $vddk_bin_path
        
        # Remove VMDK file, if it already exists
        if ( test-path $vddk_bin_path\ideDisk.vmdk )
        {
            Write-Host "$((Get-Date).ToString()): Removing existing VMDK file..."
            Remove-Item $vddk_bin_path\ideDisk.vmdk -Force
        }
        
        Write-Host "$((Get-Date).ToString()): Try creating VMDK file..."
        # Execute file and try to create a VMDK
        (& $vddk_bin_path\vmware-vdiskmanager.exe -c -s 1MB -a ide -t 0 ideDisk.vmdk 2>&1) -Replace "VixDiskLib: Invalid configuration file parameter.  Failed to read configuration file.",""
        
        # If file can not be executed
        if ( $? -eq $false )
        {
            Write-Warning "Either program could not be found, not be executed or VMDK could not be created."
            if (Test-Path "HKLM:\SOFTWARE\Classes\Installer\Products\1007C6B46D7C017319E3B52CF3EC196E" ) # Does specified registry key exist, which indicates MS Redistributable?
            {
                Write-Host "$((Get-Date).ToString()): MS Redistributable Package was found. Problem must be related to something else. Exiting."
            }
            else
            {
                Write-Host "$((Get-Date).ToString()): Could not find necessary MS Redistributable Package. Please take a look at vSphere troubleshooting part at http://wiki.sepsoftware.com. Exiting."
            }
            
            if ( $l -ne "nolog" ) # Has parameter -l been specified?
            {
                Stop-Transcript
            }
            exit 8
        }
        # Otherwise delete created file and switch back to original location
        else
        {
            Remove-Item ideDisk.vmdk
            Set-Location $currentdir
        }
        
        Write-Host "$((Get-Date).ToString()): Creation of VMDK file was successful."
        Write-Host "$((Get-Date).ToString()): Comparing VDDK libraries in sesam and VM-Ware directory."
        # Does vixDiskLib.dll already exist?
        if ( test-path "$vddk_bin_path\vixDiskLib.dll" )
        {
            Write-Host "$((Get-Date).ToString()): Found vixDiskLib.dll in $vddk_bin_path"
            # Get installed VDDK version
            $vddk = ([int](Get-Item $vddk_bin_path\vixDiskLib.dll).VersionInfo.ProductVersion.Split()[0].Replace(".",""))
            
            # If something went wrong during step before
            if ( $vddk -eq $null )
            {
                Write-Error "Error while reading meta information of vixDiskLib.dll in $vddk_bin_path"
                if ( $l -ne "nolog" ) # Has parameter -l been specified?
                {
                    Stop-Transcript
                }
                exit 4
            }
            # If VDDK 5.1.1 or 5.5.0 or 5.5.1 or 5.5.2 or 5.5.3 is installed
            if ( ($vddk -eq 511) -or ($vddk -eq 550) -or ($vddk -eq 551) -or ($vddk -eq 552) -or ($vddk -eq 553) )
            {
                        Write-Host "$((Get-Date).ToString()): VDDK 5.1.1 or 5.5.x was found."
                        $x86_vddk_bin_path = $env:ProgramFiles + "\VMware\VMware Virtual Disk Development Kit\bin"              # Set VDDK path
                        $sesam_ssleay = (Get-Item $env:sesam\ssleay32.dll).VersionInfo.ProductVersion                           # Get version of ssleay32.dll in sesam directory
                        $vddk511_ssleay = (Get-Item $x86_vddk_bin_path\ssleay32.dll).VersionInfo.ProductVersion                 # Get version of ssleay32.dll in VM-Ware directory
                        $sesam_sbc_vmware = (Get-Content $env:ini\sm.ini | Select-String version=).ToString().Split(',')[1]     # Get current sesam version
                        # Either result was empty or content is not like a Sesam version
                        if ( ( $sesam_sbc_vmware -eq $null ) -or ( $sesam_sbc_vmware -notmatch "[0-9]\.[0-9]\.[0-9]\.[0-9]" ) )
                        {
                            Write-Warning "Could not determine current sesam version. Use version of ssleay32 instead."
                            $sesam_sbc_vmware = $sesam_ssleay
                        }
                        $sesam_libeay = (Get-Item $env:sesam\libeay32.dll).VersionInfo.ProductVersion                           # Get version of libeay32.dll in sesam directory
                        
                        if ( $sesam_ssleay -ne $vddk511_ssleay )             # If versions are not equal
                        {
                            Write-Host "$((Get-Date).ToString()): ssleay32.dll in $env:sesam is version $sesam_ssleay, but version $vddk511_ssleay in $x86_vddk_bin_path. Starting sync."

                                move-item $env:sesam\ssleay32.dll $env:sesam\ssleay32_$sesam_ssleay.dll -Force               # Rename dll in sesam directory
                                if ( $? -eq $false )
                                {
                                    Write-Warning "Cannot move file `"$($env:sesam)\ssleay32.dll`", probably due to permission problems. Exiting."
                                    if ( $l -ne "nolog" ) # Has parameter -l been specified?
                                    {
                                        Stop-Transcript
                                    }
                                    exit 8
                                }
                                move-item $env:sesam\sbc_vmware.dll $env:sesam\sbc_vmware_$sesam_sbc_vmware.dll -Force       # Rename dll in sesam directory
                                if ( $? -eq $false )
                                {
                                    Write-Warning "Cannot move file `"$($env:sesam)\sbc_vmware.dll`", probably due to permission problems. Undo last steps before exiting."
                                    move-item $env:sesam\ssleay32_$sesam_ssleay.dll $env:sesam\ssleay32.dll -Force           # Rename first file back to it's original
                                    if ( $l -ne "nolog" ) # Has parameter -l been specified?
                                    {
                                        Stop-Transcript
                                    }
                                    exit 8
                                }                                   
                                move-item $env:sesam\libeay32.dll $env:sesam\libeay32_$sesam_libeay.dll -Force               # Rename dll in sesam directory
                                if ( $? -eq $false )
                                {
                                    Write-Warning "Cannot move file `"$($env:sesam)\libeay32.dll`", probably due to permission problems. Undo last steps before exiting."
                                    move-item $env:sesam\ssleay32_$sesam_ssleay.dll $env:sesam\ssleay32.dll -Force           # Rename first file back to it's original
                                    move-item $env:sesam\sbc_vmware_$sesam_sbc_vmware.dll $env:sesam\sbc_vmware.dll -Force   # Rename second file back to it's original
                                    if ( $l -ne "nolog" ) # Has parameter -l been specified?
                                    {
                                        Stop-Transcript
                                    }
                                    exit 8
                                }               
                            
                                if ( $vddk -eq 511 )      # If VDDK 5.1.1 is installed
                                {
                                    Write-Host "$((Get-Date).ToString()): Use vmware_vddk5.1.1.zip for extract."
                                    $vddk_zip = "$env:sesam" + "vmware_vddk5.1.1.zip"               # Specify the 5.1.1 ZIP file to extract
                                }
                                elseif ( ( $vddk -eq 550 ) -or ( $vddk -eq 551 ) -or ($vddk -eq 552) -or ($vddk -eq 553) )  # If VDDK 5.5.0 or VDDK 5.5.1 or VDDK 5.5.2 or VDDK 5.5.3 is installed
                                {
                                    Write-Host "$((Get-Date).ToString()): Use vmware_vddk5.5.1.zip for extract."
                                    $vddk_zip = "$env:sesam" + "vmware_vddk5.5.0.zip"               # Specify the 5.5.0 ZIP file to extract, because libeay32 and ssleay32 are the same version as with 5.5.0. Just vixDiskib changed.
                                }
                                $destination = $env:sesam                                       # destination folder
                    		    if(test-path($vddk_zip))                                  		# Test if the zip file exists 
    		                    {    
    			                     $shellApplication = new-object -com shell.application 
    			                     $zipPackage = $shellApplication.NameSpace($vddk_zip) 
    			                     $destinationFolder = $shellApplication.NameSpace($destination)

    			                     $destinationFolder.CopyHere($zipPackage.Items(), 0x14)     # Extract files and overwrite all files without asking (0x14)
                                }
                                else 
    		                    { 
    			                    echo "The source ZIP file $vddk_zip does not exist";
                                    if ( $l -ne "nolog" ) # Has parameter -l been specified?
                                    {
                                        Stop-Transcript
                                    }
    			                    exit 3
    		                    }
                            

 
                       }
                       else
                       {
                            Write-Host "$((Get-Date).ToString()): Files in $env:sesam and in $x86_vddk_bin_path are equal, nothing to do."
                       }  

            }
            else
            {
                Write-Host "$((Get-Date).ToString()): Found version $vddk of VDDK."
            }
            
        }
        else
        {
            Write-Warning "$vddk_bin_path\vixDiskLib.dll not found, exiting."
            if ( $l -ne "nolog" ) # Has parameter -l been specified?
            {
                Stop-Transcript
            }
		    exit
        }

}


function checkvddkplatform ()
{
    # Just continue, if VDDK is not installed anymore (up from 2013-12-23 x86 is also used for x64, when VDDK 5.5.0 or higher is in use)
    if ( ! ( test-path "HKLM:\SOFTWARE\Wow6432Node\VMware, Inc.\VMware Virtual Disk Development Kit\" ) )
    {
        Write-Host "$((Get-Date).ToString()): VDDK does not exist on system (anymore) or VDDK 5.5.0 or higher is installed, checking registry path for manually created struture."
        # Just continue, if manually created structure exists
        if ( test-path "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\" )
        {
            # Get information, whether VDDK has been installed by SEP, which can be the case up from VDDK 5.5
            $vddk_bysep = ((Get-ItemProperty "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\").InstalledBySesam)
            if ( $vddk_bysep -eq $null ) # Has VDDK been installed by SEP?
            {
                Write-Host "$((Get-Date).ToString()): VDDK has been not installed by SEP."
            
                # Get directory, where vixdisklib.dll is located
                $vddk_vixlib_path = ((Get-ItemProperty "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\").InstallPath) + "bin"
                # Get installed VDDK version
                $vddk = ([int](Get-Item $vddk_vixlib_path\vixDiskLib.dll).VersionInfo.ProductVersion.Split()[0].Replace(".",""))
                
                
                if ( $vddk -ge 550 )   # If VDDK 5.5.0 or higher is installed, which does not have an installer anymore
                {
                    Write-Host "$((Get-Date).ToString()): VDDK 5.5.0 or higher is in use. Do not remove files or registry values."
                    return $false
                }  
                else
                {
                
                    # Find path of VDDK files, which were created manually
                    $vddk_files = ((Get-ItemProperty "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\").InstallPath)
                    
                    # Remove registry structure
                    Write-Host "$((Get-Date).ToString()): Removing manually created registry structure..."
                    Remove-Item "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\" -Force -Recurse
                    # Check, whether deleting was successful or not
                    if ( $? -eq $false )
                    {
                        Write-Host "$((Get-Date).ToString()): Registry path could not be deleted, exiting."
                        if ( $l -ne "nolog" ) # Has parameter -l been specified?
                        {
                            Stop-Transcript
                        }
                        exit 6
                    }
                    
                    # Remove physical files
                    Write-Host "$((Get-Date).ToString()): Removing folders and files from manually created structure..."
                    Remove-Item $vddk_files -Force -Recurse
                    # Check, whether deleting was successful or not
                    if ( $? -eq $false )
                    {
                        Write-Host "$((Get-Date).ToString()): Files could not be deleted, exiting."
                        if ( $l -ne "nolog" ) # Has parameter -l been specified?
                        {
                            Stop-Transcript
                        }
                        exit 6
                    }
                    Write-Host "$((Get-Date).ToString()): Leaving remove part."
                    return $true
                }
            }
            else
            {
                Write-Host "$((Get-Date).ToString()): VDDK has been installed by SEP, nothing to do in this case."
                return $true
            }
        }
        else
        {
            Write-Host "$((Get-Date).ToString()): No VDDK is installed yet."
            return $true
        }
    }
    else
    {
        Write-Host "$((Get-Date).ToString()): VDDK still exists in Wow6432Node."
        return $true
    }
}


function installvddkviazip ($version, $zipfilepath, $bysep, $moveoldfiles, $inipath, $sslverify)
{
    if ( new-item -ItemType directory "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit" -force )
	{
    	Write-Host "$((Get-Date).ToString()): First step of VDDK structure for version $version created successfully."
        # path of new folder
        $x86_vddk_path = $env:ProgramFiles + "\VMware\VMware Virtual Disk Development Kit\"
            
        # Create folder structure in file system
        new-item -ItemType directory $x86_vddk_path -force
        
        Write-Host "$((Get-Date).ToString()): VDDK folder successfully created."
    	
        # Create registry hive
        New-ItemProperty -path 'HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit' -name InstallPath -PropertyType String -value $x86_vddk_path -Force
        New-ItemProperty -path 'HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit' -name VerifySSLCertificates -PropertyType DWord -value $sslverify -Force
        
        Write-Host "$((Get-Date).ToString()): Default registry entries created successfully."

        if ( $bysep )   # If SEP should take care about the registry structure
        {
            Write-Host "$((Get-Date).ToString()): SEP takes care about the registry entries."
            New-ItemProperty -path 'HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit' -name InstalledBySesam -PropertyType Binary -value $true -Force
            New-ItemProperty -path 'HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit' -name VDDKVersion -PropertyType String -value $version -Force
            $sesam_sbc_vmware = (Get-Content $inipath\sm.ini | Select-String version=).ToString().Split(',')[1]     # Get current sesam version
            # Either result was empty or content is not like a Sesam version
            if ( ( $sesam_sbc_vmware -eq $null ) -or ( $sesam_sbc_vmware -notmatch "[0-9]\.[0-9]\.[0-9]\.[0-9]" ) )
            {
                Write-Host "$((Get-Date).ToString()): Either result was empty or content is not like a Sesam version. Content of variable is $sesam_sbc_vmware"
                return $false
            }
            Write-Host "$((Get-Date).ToString()): Inserting Sesam Version $sesam_sbc_vmware"
            New-ItemProperty -path 'HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit' -name SesamVersion -PropertyType String -value $sesam_sbc_vmware -Force
            
            # Create VMware structure in Wow6432Node due to VerifySSLCertificates value, which is used by VDDK
            new-item -ItemType directory 'HKLM:\SOFTWARE\Wow6432Node\VMware, Inc.\VMware Virtual Disk Development Kit' -Force
            New-ItemProperty -path 'HKLM:\SOFTWARE\Wow6432Node\VMware, Inc.\VMware Virtual Disk Development Kit' -name VerifySSLCertificates -PropertyType DWord -value $sslverify -Force
            
            # Create additional InstalledBySesam value to identify installation by SEP due to problems with old version of VDDK
            New-ItemProperty -path 'HKLM:\SOFTWARE\Wow6432Node\VMware, Inc.\VMware Virtual Disk Development Kit' -name InstalledBySesam -PropertyType Binary -value $true -Force
        }
        
        Write-Host "$((Get-Date).ToString()): All registry entries created successfully."
        
        if ( $moveoldfiles ) # Move files is just necessary if structure already exists
        {
            Write-Host "$((Get-Date).ToString()): Moving old files before copying new ones."
            $destination_moved = $env:ProgramFiles + "\VMware\VMware Virtual Disk Development Kit" + "_org"
            Write-Host "$((Get-Date).ToString()): Check, whether path $destination_moved already exists."
            if ( Test-Path $destination_moved )
            {
                Write-Host "$((Get-Date).ToString()): Path $destination_moved already exists. Delete the folder."
                Remove-Item $destination_moved -Force -Recurse
            }
            else
            {
                Write-Host "$((Get-Date).ToString()): Path $destination_moved does not exist."
            }
			Write-Host "$((Get-Date).ToString()): Renaming folder $x86_vddk_path to $destination_moved"
			Move-Item $x86_vddk_path $destination_moved -ErrorAction:SilentlyContinue -ErrorVariable moveerror
			if ( $? -eq $false ) 		# Just continue, if move was successful
			{
			     Write-Warning "$moveerror Exiting with Error 5."
			     return $false
			}
			new-item -ItemType directory $x86_vddk_path -force	# Create folder again, because original folder was renamed before
            Write-Host "$((Get-Date).ToString()): New folder created successfully."
        }
        
        Write-Host "$((Get-Date).ToString()): Prepare zip file to be extracted."
        $shellApplication = new-object -com shell.application 
        $zipPackage = $shellApplication.NameSpace($zipfilepath) 
        $destinationFolder = $shellApplication.NameSpace($x86_vddk_path)

        $destinationFolder.CopyHere($zipPackage.Items(), 0x14)     # Extract files and overwrite all files without asking (0x14)
        
        Write-Host "$((Get-Date).ToString()): Files extracted."
        Write-Host "$((Get-Date).ToString()): Starting installation of Vstor2 API."
        $currentdir = (pwd).Path
        Set-Location "$($env:ProgramFiles)\VMware\VMware Virtual Disk Development Kit\bin"
        (& "$($env:ProgramFiles)\VMware\VMware Virtual Disk Development Kit\bin\vstor2install.bat" )
        Set-Location $currentdir
        return $true
    }
    else
    {
        Write-Warning "Registry structure could not be created, exiting."
        return $false
    }
}


function uninstalloldvddk ()
{
            # Get directory, where vixdisklib.dll is located
            $vddk_vixlib_path = ((Get-ItemProperty "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\").InstallPath) + "bin"
            # Get installed VDDK version
            $vddk = ([int](Get-Item $vddk_vixlib_path\vixDiskLib.dll).VersionInfo.ProductVersion.Split()[0].Replace(".",""))
            
            if ( $vddk -lt 550 )
            {
                Write-Host "$((Get-Date).ToString()): Found VDDK older than 5.5. Checking OS."
                $winver = [double] ((Get-WmiObject -Class Win32_OperatingSystem).Version).SubString(0,3)   # Get first two decimals of windows version (i.e. 6.1 for Windows 7)
                $win2003r2 = (get-itemproperty -Path "HKLM:\SOFTWARE\Microsoft\Windows NT\CurrentVersion" -Name ProductName).ProductName
                
                if ( ( $winver -lt 6.0 ) )   # If system is less Windows Vista or Windows 2008
                {
                    Write-Warning "The current Windows version is too old for VDDK 5.5. Please use a newer operating system. Exiting."
                    return $false
                }
                else
                {
                    Write-Host "$((Get-Date).ToString()): Found Windows version $($winver), which is $($win2003r2). Checking VDDK via MSI."
                    $installed = ""
                    $msivddk = (Get-WmiObject -Class Win32_Product -ErrorAction SilentlyContinue -ErrorVariable installed -Filter "name = 'VMware Virtual Disk Development Kit'")  # Get information of VDDK, if installed via MSI
                    
                    if ( ( $msivddk -eq $null ) -or ( ($msivddk).Version -notmatch "^[0-9]{1}\.[0-9]{1}\.[0-9]{1}" ) )  # NULL could happen, if the WMI Class Win32_Product is not available or product was not installed via MSI, but manually.
                    {
                        Write-Warning "Could not determine VDDK version via MSI. This could happen due to missing WMI class `"Win32_Product`" or if old VDDK has been installed manually without MSI."
                        Write-Warning "In this case uninstall current VDDK manually via Windows and restart script."
                        return $false
                    }
                    else
                    {
                        Write-Host "$((Get-Date).ToString()): Version $(($msivddk).Version) found, start converting for next step."
                        $msivddk.Version = [double] (($msivddk).Version).SubString(0,3)  # Just use the first three characters of the version (i.e. 5.1)
                    }
                    
                    if ( $installed -notmatch "" )  # If an error occured during WMI query
                    {
                        Write-Warning "An error occured during WMI query. The error was:"
                        Write-Host $installed
                        Write-Host "$((Get-Date).ToString()): Exiting."
                        return $false
                    }
                    elseif ( (($msivddk).Version -ge 5) -and (($msivddk).Version -le 5.1) ) # If MSI version of VDDK is between 5.0 and 5.1
                    {
                        Write-Host "$((Get-Date).ToString()): VDDK 5.0 or 5.1 found. SEP will automatically uninstall it now."
                        Write-Host "$((Get-Date).ToString()): Found version $(($msivddk).Version) of VDDK via MSI with IdentifyingNumber $(($msivddk).IdentifyingNumber), which is used for uninstall."
                        Start-Process -FilePath msiexec -ArgumentList "/X$($msivddk.IdentifyingNumber)", "/qn" -Wait       # Start uninstall process via MSI in sync mode to wait until it finishes
                    }
                    else   # In all other cases
                    {
                        Write-Host "$((Get-Date).ToString()): Probably older version of VDDK is in use. Found version is ($msivddk).Version."
                        return $true
                    }
                    
                }
            }
            else
            {
                Write-Host "$((Get-Date).ToString()): Version of VDDK is $vddk, nothing to do in this case."
                return $true
            }
}


function updatevddk()
{
        $bysep = ((Get-ItemProperty "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\" -ErrorAction SilentlyContinue).InstalledBySesam)
        if ( ( $bysep -ne $null ) -and ( $bysep -eq 1 ) )  # Has VDDK been installed by SEP?
        {
            Write-Host "$((Get-Date).ToString()): SEP structure hive found."
            $vddkfromreg = ((Get-ItemProperty "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\" -ErrorAction SilentlyContinue).VDDKVersion) # Get VDDK version via registry
            if ( ( $vddkfromreg -ne $null ) -and ( $vddkfromreg -match "^[0-9]{3}$" ) -and ( $vddkfromreg -ge 550 ) )    # Does registry hive exist, does it contain 3 digits and is it greater or equal 550?
            {
                Write-Host "$((Get-Date).ToString()): Found VDDK version $($vddkfromreg) via registry."
                $vddk_bin_path = ((Get-ItemProperty "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\" -ErrorAction SilentlyContinue).InstallPath) + "bin"
                if ( test-path "$vddk_bin_path\vixDiskLib.dll" )  # Does vixDisklib.dll exist in bin directory?
                {
                    Write-Host "$((Get-Date).ToString()): vixDiskLib.dll found in VMware bin directory."
                    $vddkzipfile = (Get-ChildItem $env:skel\VMware-vix-disklib*.zip | Sort-Object LastWriteTime | Select-Object -Last 1).Fullname # Get youngest VMware zip file in skel folder
                    if ( $vddkzipfile -eq $null )   # Has the file been found?
                    {
                        Write-Host "$((Get-Date).ToString()): No file could be found in Sesam skel directory. Exiting."
                        if ( $l -ne "nolog" ) # Has parameter -l been specified?
                        {
                            Stop-Transcript
                        }
                        exit
                    }
                    else
                    {
                        Write-Host "$((Get-Date).ToString()): Found zip file $($vddkzipfile)."
                        $vddkzipversionarray = ($vddkzipfile).Split("-") # Split name of zip to extract version
                        $count = 0
                        while ( $count -lt $vddkzipversionarray.Length )  # Check whole array
                        {
                            Write-Host "$((Get-Date).ToString()): Checking line $($count), which contains $($vddkzipversionarray[$count])"
                            if ( $vddkzipversionarray[$count] -match "^[5-9]{1}\.[0-9]{1}\.[0-9]{1}" ) # Does part of array contain expected version string?
                            {
                                Write-Host "$((Get-Date).ToString()): Found valid version string."
                                $vddkzipversion = (($vddkzipversionarray[$count]).Replace(".","")) -Replace "x.*",""  # Remove all dots
                                Write-Host "$((Get-Date).ToString()): Version string is $vddkzipversion."
                                if ( [int] $vddkzipversion -eq [int] $vddkfromreg ) # Is zip version equal with installed version?
                                {
                                    Write-Host "$((Get-Date).ToString()): Version of zip file is equal with installed version. Exiting."
                                    if ( $l -ne "nolog" ) # Has parameter -l been specified?
                                    {
                                        Stop-Transcript
                                    }
                                    exit
                                }
                                elseif ([int] $vddkzipversion -gt [int] $vddkfromreg ) # Is zip file version greater than installed version?
                                {
                                    Write-Host "$((Get-Date).ToString()): Version of zip file is greater than installed version."
                                    $continue = $false # Reset variable for next step
                                    $continue = installvddkviazip $vddkzipversion $vddkzipfile $true $true $env:ini $sslkey
                                    if ($continue -eq $false)
                                    {
                                        Write-Warning "Something went wrong during update from VDDK version $($vddkzipversion) to version $($vddkfromreg), exiting."
                                        if ( $l -ne "nolog" ) # Has parameter -l been specified?
                                        {
                                            Stop-Transcript
                                        }
                                        exit
                                    }
                                }
                                else # In this case zip file must be less than installed version
                                {
                                    Write-Host "$((Get-Date).ToString()): Version of zip file is less than installed version exiting."
                                    if ( $l -ne "nolog" ) # Has parameter -l been specified?
                                    {
                                        Stop-Transcript
                                    }
                                    exit
                                }
                            }
                            $count++
                        }
                        
                    }
                }
                else
                {
                    Write-Host "$((Get-Date).ToString()): vixDiskLib.dll could not be found in VMware bin directory. Exiting"
                    if ( $l -ne "nolog" ) # Has parameter -l been specified?
                    {
                        Stop-Transcript
                    }
                    exit
                }
            }
            else
            {
                Write-Host "$((Get-Date).ToString()): Either VDDK version from registry is empty or format is not correct. Exiting."
                if ( $l -ne "nolog" ) # Has parameter -l been specified?
                {
                    Stop-Transcript
                }
                exit
            }
        }
        else
        {
            Write-Host "$((Get-Date).ToString()): VDDK has not been installed manually. Please update VDDK manually. Exiting."
            if ( $l -ne "nolog" ) # Has parameter -l been specified?
            {
                Stop-Transcript
            }
            exit
        }
}

function getvddkversion($installpath)
{
    $vddk = ([int](Get-Item $installpath).VersionInfo.ProductVersion.Split()[0].Replace(".",""))
    if ( $vddk -le 511 )
    {
        Write-Host "$((Get-Date).ToString()): Found version $vddk of VDDK. Checking OS."
        $winver = [double] ((Get-WmiObject -Class Win32_OperatingSystem).Version).SubString(0,3)
        $winname = (get-itemproperty -Path "HKLM:\SOFTWARE\Microsoft\Windows NT\CurrentVersion" -Name ProductName).ProductName
        if ( $winver -lt 6.0 )
        {
            Write-Host "$((Get-Date).ToString()): Found $winname, which is version $winver."
            if ( $l -ne "nolog" ) # Has parameter -l been specified?
            {
                Stop-Transcript
            }
            exit 2
        }
        else
        {
            Write-Host "$((Get-Date).ToString()): Found $winname, which is version $winver."
            if ( $l -ne "nolog" ) # Has parameter -l been specified?
            {
                Stop-Transcript
            }
            exit 1
        }
    }
    else
    {
        Write-Host "$((Get-Date).ToString()): Found version $vddk of VDDK."
        if ( $l -ne "nolog" ) # Has parameter -l been specified?
        {
            Stop-Transcript
        }
        exit 0
    }
}

if ( $c.IsPresent -eq $true ) # if -c parameter has been specified
{
    Write-Host "$((Get-Date).ToString()): Parameter `"-c`" has been specified."
    Write-Host "$((Get-Date).ToString()): Check for regular hive."
    $regularhive=((Get-ItemProperty "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\" -ErrorAction SilentlyContinue).InstallPath)
    if ( $regularhive -eq $null ) # If variable is empty
    {
        Write-Host "$((Get-Date).ToString()): No InstallPath was found. Either no VDDK is installed or VDDK has been installed manually. Checking of WOW6432Node hive follows."
    }
    else
    {
        $vixdisklib=$regularhive + "bin\vixDiskLib.dll"
        Write-Host "$((Get-Date).ToString()): Found InstallPath. Checking physical path."
        if ( Test-Path $vixdisklib )
        {
            Write-Host "$((Get-Date).ToString()): Physical path exists. Calling function to get version of VDDK."
            getvddkversion $vixdisklib
        }
        else
        {
            Write-Host "$((Get-Date).ToString()): Physical path does not exist, but registry hive. Exiting."
            if ( $l -ne "nolog" ) # Has parameter -l been specified?
            {
                Stop-Transcript
            }
            exit 1
        }
    }
    
    
    Write-Host "$((Get-Date).ToString()): Check for WOW6432 hive."
    $wow6432hive=((Get-ItemProperty "HKLM:\SOFTWARE\Wow6432Node\VMware, Inc.\VMware Virtual Disk Development Kit\" -ErrorAction SilentlyContinue).InstallPath)
    if ( $wow6432hive -eq $null ) # If variable is empty
    {
        Write-Host "$((Get-Date).ToString()): No InstallPath was found. Either no VDDK is installed or VDDK has been installed manually."
        if ( $l -ne "nolog" ) # Has parameter -l been specified?
        {
            Stop-Transcript
        }
        exit 2
    }
    else
    {
        $vixdisklibwow6432=$wow6432hive + "bin\vixDiskLib.dll"
        Write-Host "$((Get-Date).ToString()): Found InstallPath. Checking physical path."
        if ( Test-Path $vixdisklibwow6432 )
        {
            Write-Host "$((Get-Date).ToString()): Physical path exists. Calling function to get version of VDDK."
            getvddkversion $vixdisklibwow6432
        }
        else
        {
            Write-Host "$((Get-Date).ToString()): Physical path does not exist, but registry hive. Exiting."
            if ( $l -ne "nolog" ) # Has parameter -l been specified?
            {
                Stop-Transcript
            }
            exit 1
        }
        

    }
    

}

if ( $h.IsPresent -eq $true ) # if -h parameter has been specified
{
    usage
    if ( $l -ne "nolog" ) # Has parameter -l been specified?
    {
        Stop-Transcript
    }
    exit 0
}

if ( $e -notmatch "without|install|update|remove" )
{
    Write-Warning "Invalid usage of parameter -e"
    usage
    if ( $l -ne "nolog" ) # Has parameter -l been specified?
    {
        Stop-Transcript
    }
    exit
}

# Set variables via registry
if (Test-Path "HKLM:\SOFTWARE\SEP Elektronik GmbH")
{
	$env:sesam = ((get-itemproperty 'HKLM:\SOFTWARE\SEP Elektronik GmbH\sesam').SM_BIN_SESAM)
	$env:ini = ((get-itemproperty 'HKLM:\SOFTWARE\SEP Elektronik GmbH\sesam').SM_INI).Replace("sm.ini","")
    $env:skel = ($env:sesam).Replace("bin\sesam","skel")
}

if ( $ssl -eq 0 ) # If parameter $ssl has been specified with 0, which means no verify (0 is default)
{
    Write-Host "$((Get-Date).ToString()): Do not use SSL for VDDK, which is the default."
    $sslkey = 0
}
else
{
    Write-Host "$((Get-Date).ToString()): Using SSL for VDDK, which has been specified during installation."
    $sslkey = 1
}


$continue = $true    # Variable, whether next step should be executed

if ( $e -eq "remove" )  # If script has been called with "-e remove"
{
    Write-Host "$((Get-Date).ToString()): Script has been called with `"-e remove`""
    if (test-path "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\") # If structure really exists
    {
        Write-Host "$((Get-Date).ToString()): Registry structure has been found."
        $bysep = ((Get-ItemProperty "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\" -ErrorAction SilentlyContinue).InstalledBySesam)
        if ( ( $bysep -ne $null ) -and ( $bysep -eq 1 ) )  # Has VDDK been installed by SEP?
        {
            Write-Host "$((Get-Date).ToString()): SEP structure hive found."
            Write-Host "$((Get-Date).ToString()): Starting uninstall process of Vstor2 API."
            & "$($env:ProgramFiles)\VMware\VMware Virtual Disk Development Kit\bin\vstor2uninstall.bat"
            if (test-path "$($env:ProgramFiles)\VMware\VMware Virtual Disk Development Kit\") # Does file structure still exist?
            { 
                Write-Host "$((Get-Date).ToString()): Start removing file structure."
                Remove-Item "$($env:ProgramFiles)\VMware\VMware Virtual Disk Development Kit\" -Force -Recurse
            }
            Write-Host "$((Get-Date).ToString()): Start removing registry structure."
            Remove-Item "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\" -Force -Recurse
            Write-Host "$((Get-Date).ToString()): Start removing registry structure in Wow6432Node hive."
            Remove-Item "HKLM:\SOFTWARE\Wow6432Node\VMware, Inc.\VMware Virtual Disk Development Kit\" -Force -Recurse -ErrorAction SilentlyContinue
            Write-Host "$((Get-Date).ToString()): Removing done."
        }
        else
        {
            Write-Host "$((Get-Date).ToString()): VDDK 5.5 or higher has been installed manually. Please uninstall it manually."
            if ( $l -ne "nolog" ) # Has parameter -l been specified?
            {
                Stop-Transcript
            }
            exit
        }
    }
    else
    {
        Write-Host "$((Get-Date).ToString()): No VDDK structure was found"
        if ( $l -ne "nolog" ) # Has parameter -l been specified?
        {
            Stop-Transcript
        }
        exit
    }
    if ( $l -ne "nolog" ) # Has parameter -l been specified?
    {
        Stop-Transcript
    }
    exit
}

if ( $e -eq "update" )
{
    Write-Host "$((Get-Date).ToString()): Script has been called with `"-e update`""
    if (test-path "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\") # If structure really exists
    {
        Write-Host "$((Get-Date).ToString()): Registry structure has been found."
        updatevddk # Call function, which tries to update VDDK structure
    }
    else
    {
        Write-Host "$((Get-Date).ToString()): No VDDK has been found. Update not possible. Exiting."
        if ( $l -ne "nolog" ) # Has parameter -l been specified?
        {
            Stop-Transcript
        }
        exit
    }
}

Write-Host "$((Get-Date).ToString()): Check for 32 or 64 Bit..."
# Check, whether system is x64
if ( test-path "HKLM:\SOFTWARE\Wow6432Node\" )
{
    Write-Host "$((Get-Date).ToString()): System is x64"
    if ( $e -eq "install" ) # Just continue at this part, if "-e install" has been specified when calling the script
    {
        Write-Host "$((Get-Date).ToString()): Switch `"-e install`" has been specified to install new VDDK, if older version is installed. Checking for zip file."
        $vddkzipfile = "" # Set variable for next step
        $vddkzipfile = (Get-ChildItem $env:skel\VMware-vix-disklib*.zip | Sort-Object LastWriteTime | Select-Object -Last 1).Fullname # Get youngest VMware zip file in skel folder
        if ( $vddkzipfile -eq $null )   # Has the file been found?
        {
            Write-Warning "$((Get-Date).ToString()): No file could be found in Sesam skel directory. Exiting."
            if ( $l -ne "nolog" ) # Has parameter -l been specified?
            {
                Stop-Transcript
            }
            exit
        }
        else
        {
            Write-Host "$((Get-Date).ToString()): Found zip file $vddkzipfile."
            if ( ( test-path "HKLM:\SOFTWARE\Wow6432Node\VMware, Inc.\VMware Virtual Disk Development Kit\" ) -and ( test-path "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\" ) ) # Check for old VDDK (>=5.1.1), which was installed on this path. Now new checks have been implemented due to manual value in WOW64632
            {
                Write-Host "$((Get-Date).ToString()): Old VDDK is installed, checking version and removing it, if necessary."
                $vddk_by_sesam = ((Get-ItemProperty "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\").InstalledBySesam) # Check value "InstalledBySesam", which indicates installation of VDDK by SEP
                if ( $vddk_by_sesam -eq $true )
                {
                    Write-Host "$((Get-Date).ToString()): VDDK in regular hive has been installed by SEP."
                    if ( $f.IsPresent -eq $true ) # If -f has been specified
                    {
                        Write-Host "$((Get-Date).ToString()): Parameter -f has been specified."
                    }
                    else
                    {
                        Write-Warning "$((Get-Date).ToString()): Script execution has not been forced, but VDDK has already been installed by SEP. Retry with -f parameter to force execution. Exiting."
                        if ( $l -ne "nolog" ) # Has parameter -l been specified?
                        {
                            Stop-Transcript
                        }
                        exit
                    }
                }
                else
                {
                    Write-Host "$((Get-Date).ToString()): VDDK in regular hive has not been installed by SEP."
                }
                $vddk_by_sesam_WOW = ((Get-ItemProperty "HKLM:\SOFTWARE\Wow6432Node\VMware, Inc.\VMware Virtual Disk Development Kit\").InstalledBySesam) # Check value "InstalledBySesam" in Wow6432Node, which indicates installation of VDDK by SEP
                if ( $vddk_by_sesam_WOW -eq $true )
                {
                    Write-Host "$((Get-Date).ToString()): VDDK in Wow6432Node hive has been installed by SEP. Install parameter may not valid in this case."
                    if ( $f.IsPresent -eq $true ) # If -f has been specified
                    {
                        Write-Host "$((Get-Date).ToString()): Parameter -f has been specified."
                    }
                    else
                    {
                        Write-Host "$((Get-Date).ToString()): Script execution has not been forced, but VDDK has already been installed by SEP. Retry with -f parameter to force execution. Exiting."
                        if ( $l -ne "nolog" ) # Has parameter -l been specified?
                        {
                            Stop-Transcript
                        }
                        exit
                    }
                }
                else
                {
                    Write-Host "$((Get-Date).ToString()): VDDK in Wow6432Node hive has not been installed by SEP."
                }
                
                if ( $f.IsPresent -eq $false ) # If -f has not been specified
                {
                    Write-Host "$((Get-Date).ToString()): Execution of script has not been forced."
                    $continue = $false    # Variable, whether next step should be executed
                    $continue = uninstalloldvddk # Call function to uninstall old VDDK
                }
                else
                {
                    Write-Host "$((Get-Date).ToString()): Execution of script has been forced with parameter -f"
                    $continue = $true    # Variable, whether next step should be executed    
                }
                
                if ($continue -eq $false)
                {
                    Write-Warning "Something went wrong during uninstall of old VDDK, exiting."
                    if ( $l -ne "nolog" ) # Has parameter -l been specified?
                    {
                        Stop-Transcript
                    }
                    exit
                }
            }
            elseif ( test-path "HKLM:\SOFTWARE\Wow6432Node\VMware, Inc.\VMware Virtual Disk Development Kit\" ) # If just Wow6432Node entry exists. This indicates a manuall installation of VDDK without SEP.
            {
                Write-Host "$((Get-Date).ToString()): Checking installed version of VDDK in Wow6432Node hive."
                $vddk_by_sesam_WOW = ((Get-ItemProperty "HKLM:\SOFTWARE\Wow6432Node\VMware, Inc.\VMware Virtual Disk Development Kit\").InstalledBySesam) # Check value "InstalledBySesam" in Wow6432Node, which indicates installation of VDDK by SEP
                if ( $vddk_by_sesam_WOW -eq $null )
                {
                    Write-Warning "$((Get-Date).ToString()): VDDK in Wow6432Node hive has not been installed by SEP. Please delete registry key and try again. Exiting."
                    if ( $l -ne "nolog" ) # Has parameter -l been specified?
                    {
                        Stop-Transcript
                    }
                    exit
                }
            }
        }
    Write-Host "$((Get-Date).ToString()): Check Platform."
    $continue = $false    # Variable, whether next step should be executed
    $continue = checkvddkplatform    # Call function
    }
}

if ( ($f.IsPresent -eq $false) -and ($continue -eq $false) )   # Exit program, if no further steps are in progress and if -f was not used
{
    Write-Host "$((Get-Date).ToString()): Nothing to do anymore, exiting."
    if ( $l -ne "nolog" ) # Has parameter -l been specified?
    {
        Stop-Transcript
    }
    exit
}


Write-Host "$((Get-Date).ToString()): Continue with further steps..."


$vddkzipfile = "" # Set variable for next step
$vddkzipfile = (Get-ChildItem $env:skel\VMware-vix-disklib*.zip | Sort-Object LastWriteTime | Select-Object -Last 1).Fullname # Get youngest VMware zip file in skel folder
if ( $vddkzipfile -eq $null )   # Has the file been found?
{
    Write-Host "$((Get-Date).ToString()): No file could be found in Sesam skel directory."
}
else
{
    if ( Test-Path $vddkzipfile )   # Try to find VDDK 5.5 package in Sesam skel directory (file included on 64 bit operating systems only)
    {
        Write-Host "$((Get-Date).ToString()): VDDK file $vddkzipfile found."
        $vddkzipversionarray = ($vddkzipfile).Split("-") # Split name of zip to extract version
        $count = 0
        while ( $count -lt $vddkzipversionarray.Length )  # Check whole array
        {
            Write-Host "$((Get-Date).ToString()): Checking line $($count), which contains $($vddkzipversionarray[$count])"
            if ( $vddkzipversionarray[$count] -match "^[5-9]{1}\.[0-9]{1}\.[0-9]{1}" ) # Does part of array contain expected version string?
            {
                "$((Get-Date).ToString()): Found valid version string."
                $vddkzipversion = ($vddkzipversionarray[$count]).Replace(".","").Substring(0,3)  # Remove all dots and just use the first three digits.
            }
            $count++
        }
        
        if ( test-path "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\" )   # Test for installed VDDK via registry
        {
                Write-Host "$((Get-Date).ToString()): Checking installed version of VDDK."
                if ($f.IsPresent -eq $true)  # If -f was used when calling the script
                {
                    Write-Host "$((Get-Date).ToString()): Forcing execution of script because of -f parameter."
                    $continue = $false    # Variable, whether next step should be executed
                    $continue = uninstalloldvddk # Call function to uninstall if old VDDK is installed.
                    if ($continue -eq $false)
                    {
                        Write-Warning "Something went wrong during uninstall of old VDDK, exiting."
                        if ( $l -ne "nolog" ) # Has parameter -l been specified?
                        {
                            Stop-Transcript
                        }
                        exit
                    }
                    Write-Host "$((Get-Date).ToString()): Check Platform."
                    $continue = $false    # Variable, whether next step should be executed
                    $continue = checkvddkplatform    # Call function
                    if ( ($f.IsPresent -eq $false) -and ( $continue -eq $false) )   # Exit program, if no further steps are in progress
                    {
                        Write-Host "$((Get-Date).ToString()): Nothing to do anymore, exiting."
                        if ( $l -ne "nolog" ) # Has parameter -l been specified?
                        {
                            Stop-Transcript
                        }
                        exit
                    }
                    Write-Host "$((Get-Date).ToString()): Uninstall successful. Starting installation of VDDK 5.5"
                    $continue = $false    # Variable, whether next step should be executed
                    $continue = installvddkviazip $vddkzipversion $vddkzipfile $true $true $env:ini $sslkey    # Call function to install VDDK 5.5 from zip file
                    if ( ($f.IsPresent -eq $false) -and ($continue -eq $false) )   # Exit program, if no further steps are in progress and if -f was not used
                    {
                        Write-Host "$((Get-Date).ToString()): Nothing to do anymore, exiting."
                        if ( $l -ne "nolog" ) # Has parameter -l been specified?
                        {
                            Stop-Transcript
                        }
                        exit
                    }
                }
                else
                {
                    Write-Host "$((Get-Date).ToString()): Execution of script was not forced."
                }

        }
        elseif ( $e -eq "install" ) # Just install VDDK 5.5, if "-e install" has been specified
        {
            Write-Host "$((Get-Date).ToString()): No VDDK has been found on your system. SEP starts installing version 5.5"
            $continue = $false    # Variable, whether next step should be executed
            $continue = installvddkviazip $vddkzipversion $vddkzipfile $true $false $env:ini $sslkey     # Call function to install VDDK 5.5 from zip file
            if ( ($f.IsPresent -eq $false) -and ($continue -eq $false) )   # Exit program, if no further steps are in progress and if -f was not used
                {
                    Write-Host "$((Get-Date).ToString()): Nothing to do anymore, exiting."
                    if ( $l -ne "nolog" ) # Has parameter -l been specified?
                    {
                        Stop-Transcript
                    }
                    exit
                }
        }
    }
}

# Check for x86 hive (up from 2013-12-23 x86 is also used for x64, when VDDK 5.5 or higher is in use)
if ( test-path "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit" )
{
    Write-Host "$((Get-Date).ToString()): Found VDDK structure in regular hive."
    # Get directory of VDDK
    $x86_vddk_bin_path = ((Get-ItemProperty "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\").InstallPath) + "bin"
    
    # Check for x64 hive
    if ( test-path "HKLM:\SOFTWARE\Wow6432Node\VMware, Inc.\VMware Virtual Disk Development Kit" )
    {
        Write-Host "$((Get-Date).ToString()): Found Wow6432Node hive, checking install value."
        $installed_by_sep = ((Get-ItemProperty "HKLM:\SOFTWARE\Wow6432Node\VMware, Inc.\VMware Virtual Disk Development Kit\").InstalledBySesam)
        if ( $installed_by_sep -eq $null ) # Check, whether VDDK in Wow6432Node has been installed by SEP
        {
            "$((Get-Date).ToString()): No install value has been found in Wow6432Node hive."    
            Write-Host "$((Get-Date).ToString()): Call function copy_vddk_files"
            # Call function copy_vddk_files
            copy_vddk_files
        }
        Write-Host "$((Get-Date).ToString()): Call function copy_sesam_libraries"
        # Call function copy_sesam_libraries
        copy_sesam_libraries ($x86_vddk_bin_path)
    }
    else   # VDDK is installed on an x86 system (up from 2013-12-23 x86 is also used for x64, when VDDK 5.5.0 or higher is in use)
    {
        Write-Host "$((Get-Date).ToString()): Call function copy_sesam_libraries"
        # Call function copy_sesam_libraries
        copy_sesam_libraries ($x86_vddk_bin_path)
    }
}
# Check for x64 hive and continue. Should just be called when executing script the first time at all
elseif ( test-path "HKLM:\SOFTWARE\Wow6432Node\VMware, Inc.\VMware Virtual Disk Development Kit" )
{
        Write-Host "$((Get-Date).ToString()): Just x64 data structure exists, x86 will be created now, if necessary."
        $vddk_package_code = ((Get-ItemProperty "HKLM:\SOFTWARE\WOW6432Node\VMware, Inc.\VMware Virtual Disk Development Kit\").PackageCode) # Try to get package code of VDDK
        
        if ( $vddk_package_code -eq $null ) # Is old VDDK (5.0.2 or 5.1.1) installed or has VDDK structure of VDDK been created manually?
        {
            Write-Warning "$((Get-Date).ToString()): No package code has been found in WOW6432Node hive, which indicates a manual installation of VDDK 5.5. Remove structure and try again. Exiting."
            if ( $l -ne "nolog" ) # Has parameter -l been specified?
            {
                Stop-Transcript
            }
            exit
        }
        else
        {
            # Get directory of VDDK
            $x64_vddk_bin_path = ((Get-ItemProperty "HKLM:\SOFTWARE\WOW6432Node\VMware, Inc.\VMware Virtual Disk Development Kit\").InstallPath) + "bin"
        
            Write-Host "$((Get-Date).ToString()): Found install path of VDDK in WOW6432Node hive."
            # Call function copy_vddk_files
            copy_vddk_files
		
		    # Get path of x64 files
		    $x86_vddk_bin_path = ((Get-ItemProperty "HKLM:\SOFTWARE\VMware, Inc.\VMware Virtual Disk Development Kit\").InstallPath) + "bin"
		
            Write-Host "$((Get-Date).ToString()): Call function copy_sesam_libraries"
            # Call function copy_sesam_libraries
            copy_sesam_libraries ($x86_vddk_bin_path)
        }
}


# Otherwise product seems not to be installed or script was executed on an x86 system
else
{
	Write-Host "$((Get-Date).ToString()): No 'VMware Virtual Disk Development Kit' found"
    usage
    if ( $l -ne "nolog" ) # Has parameter -l been specified?
    {
        Stop-Transcript
    }
	exit 0
}


if ( $l -ne "nolog" ) # Has parameter -l been specified?
{
    Stop-Transcript
}
# SIG # Begin signature block
# MIIZAgYJKoZIhvcNAQcCoIIY8zCCGO8CAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQU6QSVaNe+gkF4fd9gcvmQ7EhO
# QzWgghP6MIID7jCCA1egAwIBAgIQfpPr+3zGTlnqS5p31Ab8OzANBgkqhkiG9w0B
# AQUFADCBizELMAkGA1UEBhMCWkExFTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTEUMBIG
# A1UEBxMLRHVyYmFudmlsbGUxDzANBgNVBAoTBlRoYXd0ZTEdMBsGA1UECxMUVGhh
# d3RlIENlcnRpZmljYXRpb24xHzAdBgNVBAMTFlRoYXd0ZSBUaW1lc3RhbXBpbmcg
# Q0EwHhcNMTIxMjIxMDAwMDAwWhcNMjAxMjMwMjM1OTU5WjBeMQswCQYDVQQGEwJV
# UzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9yYXRpb24xMDAuBgNVBAMTJ1N5bWFu
# dGVjIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0EgLSBHMjCCASIwDQYJKoZIhvcN
# AQEBBQADggEPADCCAQoCggEBALGss0lUS5ccEgrYJXmRIlcqb9y4JsRDc2vCvy5Q
# WvsUwnaOQwElQ7Sh4kX06Ld7w3TMIte0lAAC903tv7S3RCRrzV9FO9FEzkMScxeC
# i2m0K8uZHqxyGyZNcR+xMd37UWECU6aq9UksBXhFpS+JzueZ5/6M4lc/PcaS3Er4
# ezPkeQr78HWIQZz/xQNRmarXbJ+TaYdlKYOFwmAUxMjJOxTawIHwHw103pIiq8r3
# +3R8J+b3Sht/p8OeLa6K6qbmqicWfWH3mHERvOJQoUvlXfrlDqcsn6plINPYlujI
# fKVOSET/GeJEB5IL12iEgF1qeGRFzWBGflTBE3zFefHJwXECAwEAAaOB+jCB9zAd
# BgNVHQ4EFgQUX5r1blzMzHSa1N197z/b7EyALt0wMgYIKwYBBQUHAQEEJjAkMCIG
# CCsGAQUFBzABhhZodHRwOi8vb2NzcC50aGF3dGUuY29tMBIGA1UdEwEB/wQIMAYB
# Af8CAQAwPwYDVR0fBDgwNjA0oDKgMIYuaHR0cDovL2NybC50aGF3dGUuY29tL1Ro
# YXd0ZVRpbWVzdGFtcGluZ0NBLmNybDATBgNVHSUEDDAKBggrBgEFBQcDCDAOBgNV
# HQ8BAf8EBAMCAQYwKAYDVR0RBCEwH6QdMBsxGTAXBgNVBAMTEFRpbWVTdGFtcC0y
# MDQ4LTEwDQYJKoZIhvcNAQEFBQADgYEAAwmbj3nvf1kwqu9otfrjCR27T4IGXTdf
# plKfFo3qHJIJRG71betYfDDo+WmNI3MLEm9Hqa45EfgqsZuwGsOO61mWAK3ODE2y
# 0DGmCFwqevzieh1XTKhlGOl5QGIllm7HxzdqgyEIjkHq3dlXPx13SYcqFgZepjhq
# IhKjURmDfrYwggSjMIIDi6ADAgECAhAOz/Q4yP6/NW4E2GqYGxpQMA0GCSqGSIb3
# DQEBBQUAMF4xCzAJBgNVBAYTAlVTMR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3Jh
# dGlvbjEwMC4GA1UEAxMnU3ltYW50ZWMgVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBD
# QSAtIEcyMB4XDTEyMTAxODAwMDAwMFoXDTIwMTIyOTIzNTk1OVowYjELMAkGA1UE
# BhMCVVMxHTAbBgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9uMTQwMgYDVQQDEytT
# eW1hbnRlYyBUaW1lIFN0YW1waW5nIFNlcnZpY2VzIFNpZ25lciAtIEc0MIIBIjAN
# BgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAomMLOUS4uyOnREm7Dv+h8GEKU5Ow
# mNutLA9KxW7/hjxTVQ8VzgQ/K/2plpbZvmF5C1vJTIZ25eBDSyKV7sIrQ8Gf2Gi0
# jkBP7oU4uRHFI/JkWPAVMm9OV6GuiKQC1yoezUvh3WPVF4kyW7BemVqonShQDhfu
# ltthO0VRHc8SVguSR/yrrvZmPUescHLnkudfzRC5xINklBm9JYDh6NIipdC6Anqh
# d5NbZcPuF3S8QYYq3AhMjJKMkS2ed0QfaNaodHfbDlsyi1aLM73ZY8hJnTrFxeoz
# C9Lxoxv0i77Zs1eLO94Ep3oisiSuLsdwxb5OgyYI+wu9qU+ZCOEQKHKqzQIDAQAB
# o4IBVzCCAVMwDAYDVR0TAQH/BAIwADAWBgNVHSUBAf8EDDAKBggrBgEFBQcDCDAO
# BgNVHQ8BAf8EBAMCB4AwcwYIKwYBBQUHAQEEZzBlMCoGCCsGAQUFBzABhh5odHRw
# Oi8vdHMtb2NzcC53cy5zeW1hbnRlYy5jb20wNwYIKwYBBQUHMAKGK2h0dHA6Ly90
# cy1haWEud3Muc3ltYW50ZWMuY29tL3Rzcy1jYS1nMi5jZXIwPAYDVR0fBDUwMzAx
# oC+gLYYraHR0cDovL3RzLWNybC53cy5zeW1hbnRlYy5jb20vdHNzLWNhLWcyLmNy
# bDAoBgNVHREEITAfpB0wGzEZMBcGA1UEAxMQVGltZVN0YW1wLTIwNDgtMjAdBgNV
# HQ4EFgQURsZpow5KFB7VTNpSYxc/Xja8DeYwHwYDVR0jBBgwFoAUX5r1blzMzHSa
# 1N197z/b7EyALt0wDQYJKoZIhvcNAQEFBQADggEBAHg7tJEqAEzwj2IwN3ijhCcH
# bxiy3iXcoNSUA6qGTiWfmkADHN3O43nLIWgG2rYytG2/9CwmYzPkSWRtDebDZw73
# BaQ1bHyJFsbpst+y6d0gxnEPzZV03LZc3r03H0N45ni1zSgEIKOq8UvEiCmRDoDR
# EfzdXHZuT14ORUZBbg2w6jiasTraCXEQ/Bx5tIB7rGn0/Zy2DBYr8X9bCT2bW+IW
# yhOBbQAuOA2oKY8s4bL0WqkBrxWcLC9JG9siu8P+eJRRw4axgohd8D20UaF5Mysu
# e7ncIAkTcetqGVvP6KUwVyyJST+5z3/Jvz4iaGNTmr1pdKzFHTx/kuDDvBzYBHUw
# ggVPMIIEN6ADAgECAhBFJdVMzQeBwEJaHN/ndDHmMA0GCSqGSIb3DQEBBQUAMIG0
# MQswCQYDVQQGEwJVUzEXMBUGA1UEChMOVmVyaVNpZ24sIEluYy4xHzAdBgNVBAsT
# FlZlcmlTaWduIFRydXN0IE5ldHdvcmsxOzA5BgNVBAsTMlRlcm1zIG9mIHVzZSBh
# dCBodHRwczovL3d3dy52ZXJpc2lnbi5jb20vcnBhIChjKTEwMS4wLAYDVQQDEyVW
# ZXJpU2lnbiBDbGFzcyAzIENvZGUgU2lnbmluZyAyMDEwIENBMB4XDTExMTEyNTAw
# MDAwMFoXDTE0MTEyNDIzNTk1OVowgZIxCzAJBgNVBAYTAkRFMRAwDgYDVQQIEwdC
# YXZhcmlhMQ8wDQYDVQQHEwZXZXlhcm4xDzANBgNVBAoUBlNFUCBBRzE+MDwGA1UE
# CxM1RGlnaXRhbCBJRCBDbGFzcyAzIC0gTWljcm9zb2Z0IFNvZnR3YXJlIFZhbGlk
# YXRpb24gdjIxDzANBgNVBAMUBlNFUCBBRzCCASIwDQYJKoZIhvcNAQEBBQADggEP
# ADCCAQoCggEBAKPk8sFkobQAgR7xfCW3B7oNFbaer+Bc2MLh7Tctb+jJHBuZtczR
# D0UUsBLb8TRVnujZSxuRQSnK9vB/uaI3gy42d/+XqiwcZ92l+wcQB6tuS2FTAb1z
# d+f2irj+ewHZCrxSaWyxQLuu8TQbxhMCDvh+75UAfkJby1haO6XXqXDp28DEDYTl
# J34g3yAqmMX2pscHpnk1mDO087Sf22PZ/capDI2kPO08EijfiOPDfLxoO/gOVb9a
# iGNFU0xKoWITJuZirKwS0v2prPK2YCm0bAmcoRlnxjUfLY+rMMmK73WLTuoJrGtg
# JBHuARLK2QgbpFeo0N4ZuJPo78XiYPIAjuECAwEAAaOCAXswggF3MAkGA1UdEwQC
# MAAwDgYDVR0PAQH/BAQDAgeAMEAGA1UdHwQ5MDcwNaAzoDGGL2h0dHA6Ly9jc2Mz
# LTIwMTAtY3JsLnZlcmlzaWduLmNvbS9DU0MzLTIwMTAuY3JsMEQGA1UdIAQ9MDsw
# OQYLYIZIAYb4RQEHFwMwKjAoBggrBgEFBQcCARYcaHR0cHM6Ly93d3cudmVyaXNp
# Z24uY29tL3JwYTATBgNVHSUEDDAKBggrBgEFBQcDAzBxBggrBgEFBQcBAQRlMGMw
# JAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLnZlcmlzaWduLmNvbTA7BggrBgEFBQcw
# AoYvaHR0cDovL2NzYzMtMjAxMC1haWEudmVyaXNpZ24uY29tL0NTQzMtMjAxMC5j
# ZXIwHwYDVR0jBBgwFoAUz5mp6nsm9EvJjo/X8AUm7+PSp50wEQYJYIZIAYb4QgEB
# BAQDAgQQMBYGCisGAQQBgjcCARsECDAGAQEAAQH/MA0GCSqGSIb3DQEBBQUAA4IB
# AQDtgoI/IZxnwNdt+NbmQ8oinCBqJjsU8h4kqgI3MOEkabBZb+yZIPM5cWWnce57
# 5WtZH21wG/Gl5oaNpu8D4FgvnhHDWm9gykLCWJPUfucH8LY4xXE5eGLPQHMvslNe
# omxeJPhx0iTgiMEhxcMUcWDc8KVcC6xsS2uwezqEEv+f8ehI/Q+I8Cbw+xiifNEK
# GovEw/sbeVXgYk70MbVVA8LGBv5ZXnBTK6SLEezXCLkFkEsU8NV6Xc8oplxgDF5p
# Bh8yeZvUoPDsb0eNxwS93y4YMRQrMurCwwfXcqyisEvX+J0JsvB2VNNo4LIqgDv1
# tP5zBnSVmlFZo9eRxtRSAiQoMIIGCjCCBPKgAwIBAgIQUgDlqiVW/BqG7ZbJ1Esz
# xzANBgkqhkiG9w0BAQUFADCByjELMAkGA1UEBhMCVVMxFzAVBgNVBAoTDlZlcmlT
# aWduLCBJbmMuMR8wHQYDVQQLExZWZXJpU2lnbiBUcnVzdCBOZXR3b3JrMTowOAYD
# VQQLEzEoYykgMjAwNiBWZXJpU2lnbiwgSW5jLiAtIEZvciBhdXRob3JpemVkIHVz
# ZSBvbmx5MUUwQwYDVQQDEzxWZXJpU2lnbiBDbGFzcyAzIFB1YmxpYyBQcmltYXJ5
# IENlcnRpZmljYXRpb24gQXV0aG9yaXR5IC0gRzUwHhcNMTAwMjA4MDAwMDAwWhcN
# MjAwMjA3MjM1OTU5WjCBtDELMAkGA1UEBhMCVVMxFzAVBgNVBAoTDlZlcmlTaWdu
# LCBJbmMuMR8wHQYDVQQLExZWZXJpU2lnbiBUcnVzdCBOZXR3b3JrMTswOQYDVQQL
# EzJUZXJtcyBvZiB1c2UgYXQgaHR0cHM6Ly93d3cudmVyaXNpZ24uY29tL3JwYSAo
# YykxMDEuMCwGA1UEAxMlVmVyaVNpZ24gQ2xhc3MgMyBDb2RlIFNpZ25pbmcgMjAx
# MCBDQTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAPUjS16l14q7MunU
# V/fv5Mcmfq0ZmP6onX2U9jZrENd1gTB/BGh/yyt1Hs0dCIzfaZSnN6Oce4DgmeHu
# N01fzjsU7obU0PUnNbwlCzinjGOdF6MIpauw+81qYoJM1SHaG9nx44Q7iipPhVuQ
# AU/Jp3YQfycDfL6ufn3B3fkFvBtInGnnwKQ8PEEAPt+W5cXklHHWVQHHACZKQDy1
# oSapDKdtgI6QJXvPvz8c6y+W+uWHd8a1VrJ6O1QwUxvfYjT/HtH0WpMoheVMF05+
# W/2kk5l/383vpHXv7xX2R+f4GXLYLjQaprSnTH69u08MPVfxMNamNo7WgHbXGS6l
# zX40LYkCAwEAAaOCAf4wggH6MBIGA1UdEwEB/wQIMAYBAf8CAQAwcAYDVR0gBGkw
# ZzBlBgtghkgBhvhFAQcXAzBWMCgGCCsGAQUFBwIBFhxodHRwczovL3d3dy52ZXJp
# c2lnbi5jb20vY3BzMCoGCCsGAQUFBwICMB4aHGh0dHBzOi8vd3d3LnZlcmlzaWdu
# LmNvbS9ycGEwDgYDVR0PAQH/BAQDAgEGMG0GCCsGAQUFBwEMBGEwX6FdoFswWTBX
# MFUWCWltYWdlL2dpZjAhMB8wBwYFKw4DAhoEFI/l0xqGrI2Oa8PPgGrUSBgsexku
# MCUWI2h0dHA6Ly9sb2dvLnZlcmlzaWduLmNvbS92c2xvZ28uZ2lmMDQGA1UdHwQt
# MCswKaAnoCWGI2h0dHA6Ly9jcmwudmVyaXNpZ24uY29tL3BjYTMtZzUuY3JsMDQG
# CCsGAQUFBwEBBCgwJjAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AudmVyaXNpZ24u
# Y29tMB0GA1UdJQQWMBQGCCsGAQUFBwMCBggrBgEFBQcDAzAoBgNVHREEITAfpB0w
# GzEZMBcGA1UEAxMQVmVyaVNpZ25NUEtJLTItODAdBgNVHQ4EFgQUz5mp6nsm9EvJ
# jo/X8AUm7+PSp50wHwYDVR0jBBgwFoAUf9Nlp8Ld7LvwMAnzQzn6Aq8zMTMwDQYJ
# KoZIhvcNAQEFBQADggEBAFYi5jSkxGHLSLkBrVaoZA/ZjJHEu8wM5a16oCJ/30c4
# Si1s0X9xGnzscKmx8E/kDwxT+hVe/nSYSSSFgSYckRRHsExjjLuhNNTGRegNhSZz
# A9CpjGRt3HGS5kUFYBVZUTn8WBRr/tSk7XlrCAxBcuc3IgYJviPpP0SaHulhncyx
# kFz8PdKNrEI9ZTbUtD1AKI+bEM8jJsxLIMuQH12MTDTKPNjlN9ZvpSC9NOsm2a4N
# 58Wa96G0IZEzb4boWLslfHQOWP51G2M/zjF8m48blp7FU3aEW5ytkfqs7ZO6Xcgh
# U8KCU2OvEg1QhxEbPVRSloosnD2SGgiaBS7Hk6VIkdMxggRyMIIEbgIBATCByTCB
# tDELMAkGA1UEBhMCVVMxFzAVBgNVBAoTDlZlcmlTaWduLCBJbmMuMR8wHQYDVQQL
# ExZWZXJpU2lnbiBUcnVzdCBOZXR3b3JrMTswOQYDVQQLEzJUZXJtcyBvZiB1c2Ug
# YXQgaHR0cHM6Ly93d3cudmVyaXNpZ24uY29tL3JwYSAoYykxMDEuMCwGA1UEAxMl
# VmVyaVNpZ24gQ2xhc3MgMyBDb2RlIFNpZ25pbmcgMjAxMCBDQQIQRSXVTM0HgcBC
# Whzf53Qx5jAJBgUrDgMCGgUAoHAwEAYKKwYBBAGCNwIBDDECMAAwGQYJKoZIhvcN
# AQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUw
# IwYJKoZIhvcNAQkEMRYEFP8/y31fO2zX2Rt1FIL2+T9K2f3QMA0GCSqGSIb3DQEB
# AQUABIIBAGCdz7+xWoeTlORwWrPCeJNI1h5bmlHpP8Ev2toxqgCKnNs19/2tQGkJ
# ZwHYjTvwcfJpLBlwyBqkfMH1ZzhexJRKAAnyH3RX+O04XrKr4pSUIwfpWxm3g5Rv
# gOwmm2yQKgsnrU4xUlWJy6HLjx3j8UUX1CC4IRgxXyUuC+O4Y/1j8eMq/JMUV3Y1
# 18jTDX6PXXhQkNOZiFabnxFRaSZtoKyOZU21ILiQEWWB16gjT2TCFaskfRc1Psli
# GtsU5zOIMwQpdrGbLflxwfhDeyijwqRQi6PoFlnqpuPwYSY+sYdwUELWT7iH9500
# Say9UJeo4woeoWjK4wB8KgcImDDh4LehggILMIICBwYJKoZIhvcNAQkGMYIB+DCC
# AfQCAQEwcjBeMQswCQYDVQQGEwJVUzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9y
# YXRpb24xMDAuBgNVBAMTJ1N5bWFudGVjIFRpbWUgU3RhbXBpbmcgU2VydmljZXMg
# Q0EgLSBHMgIQDs/0OMj+vzVuBNhqmBsaUDAJBgUrDgMCGgUAoF0wGAYJKoZIhvcN
# AQkDMQsGCSqGSIb3DQEHATAcBgkqhkiG9w0BCQUxDxcNMTQxMTA1MTYwMjM3WjAj
# BgkqhkiG9w0BCQQxFgQUs2+lQHq9r30+n1Yqoj3oNL/C4IkwDQYJKoZIhvcNAQEB
# BQAEggEAZ2Q/P/ZPrckQgcAcq1DImR6mjk1t40NduXON4t8e/sUADhloFjGYwmQK
# qqAwi7gb0AbkXmLuZGLlhxgN1tAe7950T0Tm20sApGcObJJ4IpEcNFiuDpT1aBOY
# s8yafVit/lV7ZBj3rg3/xiLsOqmDMonhF8P/kGIsdzbBm92ebd3IMRfHmSKa5XfW
# arzXO6CzbpV7W69PKow8+sJSg8bLqK0uxUbOVk93DMSxJAl9G3e6gWCbtAmGzR0T
# TNjwUcNI5QD2LZEoSK1x69qMdsX4ibipokbdn5zJB4T0+ZlxgBWHz+Wd2vPPoC53
# o/HKOv07/rSNAyRNbVtmdxTNcYLxVg==
# SIG # End signature block
